/*  main_xt.c

    Main file for  kview-rgb  (X11 image/movie display tool for Karma).

    Copyright (C) 1998-2001  Richard Gooch

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*
    This Karma module will enable on-screen imaging of three arrayfiles of
    the general data structure format. The arrayfiles must contain a 2 or 3
    dimensional "Intelligent Array".
    This module runs on an X11 server.


    Written by      Richard Gooch   5-SEP-1998: Copied from kview/main_xt.c

    Updated by      Richard Gooch   6-SEP-1998: Added track display.

    Updated by      Richard Gooch   17-SEP-1998: Use SimpleMenu class name in
  resources.

    Updated by      Richard Gooch   20-SEP-1998: Added mouse-based zoom in and
  out facility.

    Updated by      Richard Gooch   15-OCT-1998: Made use of new function
  <canvas_warp_pointer>.

    Updated by      Richard Gooch   27-AUG-1999: Make <data_event_func> more
  restrictive, in order to prevent grabbing other data events.

    Updated by      Richard Gooch   30-AUG-1999: Added dirname font resource.

    Updated by      Richard Gooch   31-AUG-1999: Took account of interface
  change to function <viewimg_track _compute>. Added 'l' key binding in
  function <viewimg_position_func>.

    Updated by      Richard Gooch   2-SEP-1999: Took account of interface
  change to function <canvas_warp_pointer> and discard events.

    Updated by      Richard Gooch   9-SEP-1999: Removed track display code and
  made use of XkwNupdateTrackLabels resource for ImageDisplay widget.

    Updated by      Richard Gooch   29-SEP-1999: Took account of interface
  change to <viewimg_zoom>.

    Updated by      Richard Gooch   11-DEC-1999: Removed unnecessary externs.

    Updated by      Richard Gooch   11-MAY-2001: Changed dirname font resource.

    Last updated by Richard Gooch   3-OCT-2001: Added translations for Filewin.


*/
#include <stdio.h>
#include <stdlib.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Toggle.h>
#include <karma.h>
#define NEW_WIN_SCALE
#define X11
#include <k_event_codes.h>
#include <karma_viewimg.h>
#include <karma_foreign.h>
#include <karma_xtmisc.h>
#include <karma_conn.h>
#include <karma_ds.h>
#include <karma_im.h>
#include <karma_ic.h>
#include <karma_a.h>
#include <karma_m.h>
#include <Xkw/ImageDisplay.h>
#include <Xkw/Filewin.h>
#include "kview-rgb.h"


#define VERSION "1.1.8"


/*  Private functions  */
STATIC_FUNCTION (flag viewimg_position_func,
		 (ViewableImage vimage, double x, double y,
		  void *value, unsigned int event_code,
		  void *e_info, void **f_info,
		  double x_lin, double y_lin, unsigned int value_type,
		  double x_im, double y_im, int x_pix, int y_pix) );
STATIC_FUNCTION (flag data_event_func,
		 (void *object, multi_array *multi_desc,
		  CONST char *domain, CONST char *name) );
STATIC_FUNCTION (void unload_cbk,
		 (Widget w, XtPointer client_data, XtPointer call_data) );
STATIC_FUNCTION (void region_cbk,
		 (Widget w, XtPointer client_data, XtPointer call_data) );


/*  Private data  */
String fallback_resources[] =
{
    "Kview-rgb*pseudoColourCanvas*background:              black",
    "Kview-rgb*directColourCanvas*background:              black",
    "Kview-rgb*trueColourCanvas*background:                black",
    "Kview-rgb*pseudoColourStereoCanvas*background:        black",
    "Kview-rgb*directColourStereoCanvas*background:        black",
    "Kview-rgb*trueColourStereoCanvas*background:          black",
    "Kview-rgb*AnimateControl*positionCanvas.background:   black",
    "Kview-rgb*pseudoColourCanvas*foreground:              white",
    "Kview-rgb*directColourCanvas*foreground:              white",
    "Kview-rgb*Command*background:                         grey70",
    "Kview-rgb*Repeater*background:                        grey70",
    "Kview-rgb*Ktoggle*background:                         grey80",
    "Kview-rgb*closeButton*background:                     grey90",
    "Kview-rgb*ChoiceMenu.background:                      turquoise",
    "Kview-rgb*ExclusiveMenu.background:                   turquoise",
    "Kview-rgb*Value*background:                           #d0a0a0",
    "Kview-rgb*ImageDisplay*quit*background:               orange",
    "Kview-rgb*SimpleSlider.foreground:                    Sea Green",
    "Kview-rgb*SimpleProgressMeter.background:             turquoise3",
    "Kview-rgb*SimpleProgressMeter.foreground:             grey60",
    "Kview-rgb*ImageDisplay*trackLabel0*font:              8x13bold",
    "Kview-rgb*ImageDisplay*trackLabel1*font:              8x13bold",
    "Kview-rgb*ImageDisplay*trackLabel2*font:              8x13bold",
    "Kview-rgb*SimpleMenu*font:                            10x20",
    "Kview-rgb*ImageDisplay.Command.font:                  7x13bold",
    "Kview-rgb*ImageDisplay.applicationBox.Command.font:   7x13bold",
    "Kview-rgb*ImageDisplay.Ktoggle.font:                  7x13bold",
    "Kview-rgb*ImageDisplay.ChoiceMenu.font:               7x13bold",
    "Kview-rgb*ImageDisplay.exportMenu.menuButton.font:    7x13bold",
    "Kview-rgb*ImageDisplay.overlayMenu.menuButton.font:   7x13bold",
    "Kview-rgb*ImageDisplay*zoomMenu*Unzoom*foreground:    red",
    "Kview-rgb*SimpleSlider.borderWidth:                   0",
    "Kview-rgb*IntensityPolicy*showIntensityReset:         True",
    "Kview-rgb*IntensityPolicy*resetIntensityToggle.state: True",
    "Kview-rgb*IntensityPolicy*autoIntensityScale:         False",
    "Kview-rgb*ZoomPolicy*showMiddleZoom:                  True",
    "Kview-rgb*ZoomPolicy*middleZoomToggle.state:          True",
    "Kview-rgb*ZoomPolicy*showRightZoom:                   True",
    "Kview-rgb*ZoomPolicy*rightZoomToggle.state:           True",
    "Kview-rgb*Filewin.dirname.textSink.font:              6x13",
    "Kview-rgb*Filewin*list.translations:        #override<MouseMoved>: Set()",
    "Kview-rgb*font:                                       9x15bold",
    "Kview-rgb*borderColor:                                black",
    "Kview-rgb*background:                                 aquamarine",
    "Kview-rgb*foreground:                                 black",
    NULL
};
static XrmOptionDescRec Options[] =
{
    {"-private_cmap", ".topForm.multiCanvas.pseudoColourCanvas.forceNewCmap",
     XrmoptionNoArg, (XPointer) "True"},
    {"-num_colours", ".topForm.cmapSize", XrmoptionSepArg, (XPointer) NULL},
    {"-cmap_master", ".topForm.cmapMaster", XrmoptionSepArg, (XPointer) NULL},
    {"-fullscreen", ".topForm.fullscreen", XrmoptionNoArg, (XPointer) "True"},
    {"-create_at_init", "*createAtInit", XrmoptionNoArg, (XPointer) "True"},
    {"-verbose", "*verbose", XrmoptionNoArg, (XPointer) "True"},
};
static Widget main_shell = NULL;
static Widget image_display = NULL;
static KWorldCanvas worldcanvas = NULL;


int main (int argc, char **argv)
{
    KWorldCanvas wc_pseudo, wc_direct, wc_true;
    int argcount;
    XtAppContext app_context;
    Widget filewin, filepopup, box, w, fwbox;
    char title_name[STRING_LENGTH];
    extern Dataset red_set, green_set, blue_set;
    static String dataset_names[3] = {"Red Dataset", "Green Dataset",
				      "Blue Dataset"};
    /*static char function_name[] = "main";*/

    /*  Initialise module  */
    im_register_module_name ("kview-rgb");
    im_register_module_version_date (VERSION);
    im_register_lib_version (KARMA_VERSION);
    /*  Start up Xt  */
    main_shell = xtmisc_init_app_initialise (&app_context, "Kview-rgb",
					     Options, XtNumber (Options),
					     &argc, argv, fallback_resources,
					     XTMISC_INIT_ATT_MIN_CCELLS, 100,
					     XTMISC_INIT_ATT_COMMS_SETUP, TRUE,
					     XTMISC_INIT_ATT_BECOME_SERVER,
					     TRUE,
					     XTMISC_INIT_ATT_TITLE, title_name,
					     XTMISC_INIT_ATT_END,
					     NULL);
    xtmisc_set_icon (main_shell, ic_write_kimage_icon);
    image_display = XtVaCreateManagedWidget ("topForm",
					     imageDisplayWidgetClass,
					     main_shell,
					     XtNborderWidth, 0,
					     XkwNnumDatasets, 3,
					     XkwNdatasetNames, dataset_names,
					     XkwNnumTrackLabels, 3,
					     XkwNupdateTrackLabels, TRUE,
					     NULL);
    if ( ( w = XtNameToWidget (image_display, "filesButton") ) != NULL )
    {
	/*  Force main "Files" button to be re-labelled as "Red Set"  */
	XtVaSetValues (w, XtNlabel, "Red Set", NULL);
    }
    filepopup = XtNameToWidget (image_display, "filewinPopup");
    XtVaGetValues (filepopup,
		   XkwNfilewinWidget, &filewin,
		   XkwNboxWidget, &fwbox,
		   NULL);
    XtVaSetValues (filepopup,
		   XkwNfilenameTester, XkwFilewinStandardFileTester_nD,
		   XkwNdomainName, "REDFILE",
		   NULL);
    XtVaSetValues (filewin,
		   XkwNforwardSyntheticEvents, True,
		   XkwNtrapDirectoryDatasets, True,
		   NULL);
    w = XtVaCreateManagedWidget ("button", commandWidgetClass, fwbox,
				 XtNlabel, "Unload",
				 NULL);
    XtAddCallback (w, XtNcallback, unload_cbk, &red_set);
    w = XtNameToWidget (image_display, "izoomwinpopup0");
    XtAddCallback (w, XkwNregionCallback, region_cbk, &red_set);
    /*  Get the application box  */
    box = XtNameToWidget (image_display, "applicationBox");
    /*  Create the Green Set file selector and popup button  */
    filepopup = XtVaCreatePopupShell ("filewinPopup", filepopupWidgetClass,
				      main_shell,
				      XtNtitle,
				      "Kview-Rgb File Selector",
				      XtNx, 0,
				      XtNy, 0,
				      XkwNautoPopdown, True,
				      XkwNfilenameTester,
				      XkwFilewinStandardFileTester_nD,
				      XkwNdomainName, "GREENFILE",
				      XkwNloadFiles, TRUE,
				      NULL);
    w = XtVaCreateManagedWidget ("filesButton", commandWidgetClass, box,
				 XtNlabel, "Green Set",
				 NULL);
    XtAddCallback (w, XtNcallback, xtmisc_popup_cbk, filepopup);
    XtVaGetValues (filepopup,
		   XkwNfilewinWidget, &filewin,
		   XkwNboxWidget, &fwbox,
		   NULL);
    XtVaSetValues (filewin,
		   XkwNtrapDirectoryDatasets, True,
		   NULL);
    w = XtVaCreateManagedWidget ("button", commandWidgetClass, fwbox,
				 XtNlabel, "Unload",
				 NULL);
    XtAddCallback (w, XtNcallback, unload_cbk, &green_set);
    w = XtNameToWidget (image_display, "izoomwinpopup1");
    XtAddCallback (w, XkwNregionCallback, region_cbk, &green_set);
    /*  Create the Blue Set file selector and popup button  */
    filepopup = XtVaCreatePopupShell ("filewinPopup", filepopupWidgetClass,
				      main_shell,
				      XtNtitle,
				      "Kview-Rgb File Selector",
				      XtNx, 0,
				      XtNy, 0,
				      XkwNautoPopdown, True,
				      XkwNfilenameTester,
				      XkwFilewinStandardFileTester_nD,
				      XkwNdomainName, "BLUEFILE",
				      XkwNloadFiles, TRUE,
				      NULL);
    w = XtVaCreateManagedWidget ("filesButton", commandWidgetClass, box,
				 XtNlabel, "Blue Set",
				 NULL);
    XtAddCallback (w, XtNcallback, xtmisc_popup_cbk, filepopup);
    XtVaGetValues (filepopup,
		   XkwNfilewinWidget, &filewin,
		   XkwNboxWidget, &fwbox,
		   NULL);
    XtVaSetValues (filewin,
		   XkwNtrapDirectoryDatasets, True,
		   NULL);
    w = XtVaCreateManagedWidget ("button", commandWidgetClass, fwbox,
				 XtNlabel, "Unload",
				 NULL);
    XtAddCallback (w, XtNcallback, unload_cbk, &blue_set);
    w = XtNameToWidget (image_display, "izoomwinpopup2");
    XtAddCallback (w, XkwNregionCallback, region_cbk, &blue_set);
    /*  Register data handler  */
    ds_event_register_func (data_event_func, NULL);
    XtRealizeWidget (main_shell);
    XtVaGetValues (image_display,
		   XkwNpseudoColourCanvas, &wc_pseudo,
		   XkwNdirectColourCanvas, &wc_direct,
		   XkwNtrueColourCanvas, &wc_true,
		   NULL);
    worldcanvas = wc_direct;
    if (worldcanvas == NULL) worldcanvas = wc_true;
    if (worldcanvas == NULL) worldcanvas = wc_pseudo;
    XtVaSetValues (image_display, XkwNvisibleCanvas, worldcanvas, NULL);
    viewimg_register_position_event_func (worldcanvas, viewimg_position_func,
					  image_display);
    /*  Option processing after widgets created and realised  */
    for (argcount = 1; argcount < argc; ++argcount)
    {
	if ( (strcmp (argv[argcount], "-red") == 0) ||
	     (strcmp (argv[argcount], "-green") == 0) ||
	     (strcmp (argv[argcount], "-blue") == 0) )
	{
	    char *ptr = NULL;
	    multi_array *multi_desc;
	    char fname[STRING_LENGTH];

	    if (strcmp (argv[argcount], "-red") == 0) ptr = "REDFILE";
	    else if (strcmp (argv[argcount], "-green") == 0) ptr = "GREENFILE";
	    else if (strcmp (argv[argcount], "-blue") == 0) ptr = "BLUEFILE";
	    if (argcount + 1 >= argc)
	    {
		fprintf (stderr, "File missing for set\n");
		continue;
	    }
	    ++argcount;
	    if (strncmp (argv[argcount], "./", 2) == 0)
	    {
		strcpy (fname, argv[argcount] + 2);
	    }
	    else strcpy (fname, argv[argcount]);
	    if ( ( multi_desc =
		   foreign_guess_and_read (fname, K_CH_MAP_LOCAL,
					   FALSE, NULL,
					   FA_GUESS_READ_END) )
		 == NULL )
	    {
		fprintf (stderr, "Error reading file: \"%s\"\n", fname);
		continue;
	    }
	    ds_event_dispatch (multi_desc, ptr, fname);
	    ds_dealloc_multi (multi_desc);
	    m_clear (fname, STRING_LENGTH);  /*  Force me to take care  */
	}
	else fprintf (stderr, "Option: \"%s\" ignored\n", argv[argcount]);
    }
    XtAppMainLoop (app_context);
    return (RV_OK);
}   /*  End Function main  */


/*  Private routines follow  */

static flag viewimg_position_func (ViewableImage vimage, double x, double y,
				   void *value, unsigned int event_code,
				   void *e_info, void **f_info,
				   double x_lin, double y_lin,
				   unsigned int value_type,
				   double x_im, double y_im,
				   int x_pix, int y_pix)
/*  [PURPOSE] This routine is a position event consumer for a world canvas
    which has a number of ViewableImage objects associated with it.
    <viewimg> The active viewable image.
    <x> The horizontal world co-ordinate of the event.
    <y> The vertical world co-ordinate of the event.
    These values will have been transformed by the registered transform
    function (see  canvas_register_transform_func  ) for the associated
    world canvas.
    <value> A pointer to the data value in the viewable image corresponding
    to the event co-ordinates.
    <event_code> The arbitrary event code.
    <e_info> The arbitrary event information.
    <f_info> The arbitrary function information pointer.
    <x_lin> The linear horizontal world co-ordinate (the co-ordinate prior
    to the transform function being called).
    <y_lin> The linear vertical world co-ordinate (the co-ordinate prior
    to the transform function being called).
    <value_type> The type of the data value. This may be K_DCOMPLEX or
    K_UB_RGB.
    <x_im> The horizontal image pixel co-ordinate. 0.0 is the pixel centre.
    <y_im> The vertical image pixel co-ordinate. 0.0 is the pixel centre.
    <x_pix> The horizontal screen pixel co-ordinate.
    <y_pix> The vertical screen pixel co-ordinate.
    [RETURNS] TRUE if the event was consumed, else FALSE indicating that
    the event is still to be processed.
*/
{
    unsigned int prev_event;
    Bool state;
    Widget image_display = (Widget) *f_info;
    Widget zoom_policy;
    static unsigned int old_event = K_CANVAS_EVENT_POINTER_MOVE;
    static char function_name[] = "viewimg_position_func";

    prev_event = old_event;
    old_event = event_code;
    /*  Get the ZoomPolicy widget  */
    if ( ( zoom_policy = XtNameToWidget (image_display, "zoomPolicyPopup") )
	 == NULL )
    {
	fprintf (stderr, "Could not find ZoomPolicy popup\n");
	a_prog_bug (function_name);
    }
    switch (event_code)
    {
      case K_CANVAS_EVENT_MIDDLE_MOUSE_RELEASE:
	if (prev_event != K_CANVAS_EVENT_MIDDLE_MOUSE_CLICK) return (FALSE);
	XtVaGetValues (XtNameToWidget (zoom_policy, "form.middleZoomToggle"),
		       XtNstate, &state,
		       NULL);
	if (!state) return (FALSE);
	viewimg_zoom (vimage, x_lin, y_lin, FALSE, 2.0);
	canvas_warp_pointer (viewimg_get_worldcanvas (vimage),
			     x_lin, y_lin, TRUE, TRUE);
	break;
      case K_CANVAS_EVENT_RIGHT_MOUSE_RELEASE:
	if (prev_event != K_CANVAS_EVENT_RIGHT_MOUSE_CLICK) return (FALSE);
	XtVaGetValues (XtNameToWidget (zoom_policy, "form.rightZoomToggle"),
		       XtNstate, &state,
		       NULL);
	if (!state) return (FALSE);
	viewimg_zoom (vimage, 0.5, 0.5, TRUE, 0.5);
	canvas_warp_pointer (viewimg_get_worldcanvas (vimage),
			     x_lin, y_lin, TRUE, TRUE);
	break;
      default:
	return (FALSE);
	/*break;*/
    }
    return (TRUE);
}   /*  End Function viewimg_position_func  */

static flag data_event_func (void *object, multi_array *multi_desc,
			     CONST char *domain, CONST char *name)
/*  [SUMMARY] Process an event.
    <object> The object information pointer.
    <multi_desc> The multi_array descriptor.
    <domain> The domain from where the event was generated.
    <name> The name of the event in its domain.
    [RETURNS] TRUE if further callbacks should not be called, else FALSE.
*/
{
    unsigned int set_number = 0;
    Widget izoomwinpopup;
    Dataset *set = NULL;
    char txt[STRING_LENGTH];
    extern Dataset red_set, green_set, blue_set;

    if (strcmp (domain, "REDFILE") == 0) set_number = 0;
    else if (strcmp (domain, "GREENFILE") == 0) set_number = 1;
    else if (strcmp (domain, "BLUEFILE") == 0) set_number = 2;
    else
    {
	if (strcmp (name, "RedDataset") == 0) set_number = 0;
	else if (strcmp (name, "GreenDataset") == 0) set_number = 1;
	else if (strcmp (name, "BlueDataset") == 0) set_number = 2;
	else if (strncmp (name, "connection:", 11) == 0) set_number = 0;
	else return (FALSE);
    }
    switch (set_number)
    {
      case 0:
	set = &red_set;
	break;
      case 1:
	set = &green_set;
	break;
      case 2:
	set = &blue_set;
	break;
      default:
	set = NULL;
	break;
    }
    if ( !process_set (set, multi_desc, worldcanvas) ) return (FALSE);
    sprintf (txt, "izoomwinpopup%u", set_number);
    izoomwinpopup = XtNameToWidget (image_display, txt);
    XkwDataclipNewArray (izoomwinpopup, set->array, set->min, set->max, TRUE);
    return (TRUE);
}   /*  End Function data_event_func  */

static void unload_cbk (Widget w, XtPointer client_data, XtPointer call_data)
/*  [SUMMARY] Data unload callback.
    <w> The button widget.
    <client_data> The client data. This points to the dataset structure.
    <call_data> The world canvas.
    [RETURNS] Nothing.
*/
{
    Dataset *set = client_data;

    if (set->array != NULL) iarray_dealloc (set->array);
    set->array = NULL;
}   /*  End Function unload_cbk   */

static void region_cbk (Widget w, XtPointer client_data, XtPointer call_data)
/*  This is the region callback.
*/
{
    Dataset *set = client_data;
    DataclipRegions *regions = (DataclipRegions *) call_data;
    static char function_name[] = "region_cbk";

    if (regions->num_regions != 1)
    {
	fprintf (stderr, "num_regions: %u is not 1\n", regions->num_regions);
	a_prog_bug (function_name);
    }
    set->low_clip = regions->minima[0];
    set->high_clip = regions->maxima[0];
    process_range (set);
}   /*  End Function region_cbk   */

#ifdef OS_SunOS

static void dummy ()
/*  [PURPOSE] This routine is needed for SunOS, because of a problem with
    shared archive use. The <<_XtToolkitInitialize>> symbol is defined in
    the Xt shared archive, but it appears the shared archive is not linked for
    this application. Other applications (like <<krenzo>>) have no such
    problems. This is a truly bizzare bug.
*/
{
    XtToolkitInitialize ();
}   /*  End Function dummy  */
#endif
