/*  cont_hg_xt.c

    Source file for  xray  (Karma volume rendering tool).

    Copyright (C) 1996-2000  Richard Gooch

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Richard Gooch may be reached by email at  karma-request@atnf.csiro.au
    The postal address is:
      Richard Gooch, c/o ATNF, P. O. Box 76, Epping, N.S.W., 2121, Australia.
*/

/*
    This file contains the code necessary for creating the continuous Hot Gas
  control window.


    Written by      Richard Gooch   6-MAY-1996: Copied from main_xt.c

    Updated by      Richard Gooch   18-MAY-1996: Changed over to software
  colourmap rather than dummy <xc>-like functions.

    Updated by      Richard Gooch   23-NOV-1996: Allow visual to be NULL.

    Updated by      Richard Gooch   8-SEP-1998: Made use of new function
  <xc_get_location2>.

    Last updated by Richard Gooch   26-JAN-2000: Removed unnecessary includes.


*/
#include <stdio.h>
#include <errno.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Viewport.h>
#include <X11/Xaw/List.h>
#include <karma.h>
#ifndef X11
#  define X11
#endif
#include <karma_xtmisc.h>
#include <karma_kcmap.h>
#include <karma_xc.h>
#include <karma_a.h>
#include <Xkw/Canvas.h>
#include <Xkw/SimpleSlider.h>
#include "xraycast_3d.h"


/*  Private functions  */
STATIC_FUNCTION (void cont_hg_cmap_cbk, (Widget w, XtPointer client_data,
					 XtPointer call_data) );
STATIC_FUNCTION (void slider_cont_hg_cbk,
		 (Widget w, XtPointer client_data, XtPointer call_data) );
STATIC_FUNCTION (void cont_hg_apply_cbk, (Widget w, XtPointer client_data,
					  XtPointer call_data) );


/*  Private data  */


/*  Public functions follow  */

Widget create_cont_hg_win (Widget main_shell, Visual *visual)
/*  [SUMMARY] Create the continuous Hot Gas control window.
    <main_shell> The application main shell.
    <visual> The visual to be used. This may be NULL.
    [RETURNS] The control window popup.
*/
{
    KPixCanvas cmap_pixcanvas;
    Kdisplay dpy_handle;
    Kcolourmap cont_hg_cmap;
    int num_funcs;
    Colormap xcmap;
    Widget cont_hg_winpopup;
    Widget form, hist_cnv, cmap_cnv, close_btn, apply_btn;
    Widget alpha_sld, blank_min_sld, blank_max_sld, view, list;
    CONST char **func_names;
    Screen *screen;
    extern KPixCanvas cont_hg_hist_pixcanvas;
    extern int cont_hg_alpha_tenths, cont_hg_blank_min, cont_hg_blank_max;
    static char function_name[] = "create_cont_hg_win";

    func_names = kcmap_list_funcs ();
    for (num_funcs = 0; func_names[num_funcs] != NULL; ++num_funcs);
    cont_hg_winpopup = XtVaCreatePopupShell ("cont_hgwinpopup",
					     topLevelShellWidgetClass,
					     main_shell,
					     XtNtitle,
					     "Continuous Hot Gas Window",
					     NULL);
    form = XtVaCreateManagedWidget ("form", formWidgetClass,
				    cont_hg_winpopup,
				    XtNborderWidth, 0,
				    NULL);
    close_btn = XtVaCreateManagedWidget ("closeButton", commandWidgetClass,
					 form,
					 XtNlabel, "Close",
					 NULL);
    XtAddCallback (close_btn, XtNcallback, xtmisc_popdown_cbk,
		   cont_hg_winpopup);
    apply_btn = XtVaCreateManagedWidget ("button", commandWidgetClass, form,
					 XtNlabel, "Apply",
					 XtNfromHoriz, close_btn,
					 NULL);
    XtAddCallback (apply_btn, XtNcallback, cont_hg_apply_cbk, NULL);
    alpha_sld = XtVaCreateManagedWidget ("slider", simpleSliderWidgetClass,
					 form,
					 XtNlabel, "Alpha (1/10s)",
					 XtNfromVert, apply_btn,
					 XkwNminimum, 0,
					 XkwNmaximum, 50,
					 XkwNmodifier, 1,
					 XtNvalue, cont_hg_alpha_tenths,
					 XkwNvaluePtr, &cont_hg_alpha_tenths,
					 NULL);
    XtAddCallback (alpha_sld, XkwNvalueChangeCallback, slider_cont_hg_cbk,
		   NULL);
    view = XtVaCreateManagedWidget ("view", viewportWidgetClass, form,
				    XtNforceBars,True,
				    XtNwidth, 240,
				    XtNheight, 155,
				    XtNfromVert, alpha_sld,
				    XtNuseRight, True,
				    XtNallowVert, True,
				    NULL);
    list = XtVaCreateManagedWidget ("selector", listWidgetClass, view,
				    XtNlist, func_names,
				    XtNnumberStrings, num_funcs,
				    XtNdefaultColumns, 1,
				    XtNforceColumns, True,
				    NULL);
    blank_min_sld = XtVaCreateManagedWidget ("slider",
					     simpleSliderWidgetClass, form,
					     XtNlabel, "Blank Start",
					     XtNfromVert, view,
					     XtNwidth, 500,
					     XkwNminimum, -128,
					     XkwNmaximum, 127,
					     XkwNmodifier, 1,
					     XtNvalue, -128,
					     XkwNvaluePtr, &cont_hg_blank_min,
					     NULL);
    XtAddCallback (blank_min_sld, XkwNvalueChangeCallback, slider_cont_hg_cbk,
		   NULL);
    blank_max_sld = XtVaCreateManagedWidget ("slider",
					     simpleSliderWidgetClass, form,
					     XtNlabel, "Blank End",
					     XtNfromVert, blank_min_sld,
					     XtNwidth, 500,
					     XkwNminimum, -128,
					     XkwNmaximum, 127,
					     XkwNmodifier, 1,
					     XtNvalue, -128,
					     XkwNvaluePtr, &cont_hg_blank_max,
					     NULL);
    XtAddCallback (blank_max_sld, XkwNvalueChangeCallback, slider_cont_hg_cbk,
		   NULL);
    hist_cnv = XtVaCreateManagedWidget ("histCanvas",
					canvasWidgetClass, form,
					XtNfromVert, blank_max_sld,
					XtNwidth, 500,
					XtNheight, 200,
					XkwNsilenceUnconsumed, True,
					NULL);
    screen = XtScreen (main_shell);
    if ( (MaxCmapsOfScreen (screen) > 1) && (visual != NULL) )
    {
	/*  Should not get that awful colourmap flashing, so let's create
	    another canvas for the colourmap wedge  */
	cmap_cnv = XtVaCreateManagedWidget ("pseudocolourCanvas",
					    canvasWidgetClass, form,
					    XtNfromVert, hist_cnv,
					    XtNwidth, 500,
					    XtNheight, 100,
					    XtNvisual, visual,
					    XkwNforceNewCmap, True,
					    XkwNsilenceUnconsumed, True,
					    XtNmappedWhenManaged, True,
					    NULL);
    }
    else cmap_cnv = NULL;
    XtRealizeWidget (cont_hg_winpopup);
    XtVaGetValues (hist_cnv,
		   XkwNmonoPixCanvas, &cont_hg_hist_pixcanvas,
		   NULL);
    if (cmap_cnv == NULL)
    {
	if ( ( cont_hg_cmap = kcmap_va_create (DEFAULT_COLOURMAP_NAME,
					       FULLCOLOUR_HOT_GAS_CMAP_SIZE,
					       FALSE, NULL,
					       ( unsigned int (*) () ) NULL,
					       ( void (*) () ) NULL,
					       ( void (*) () ) NULL,
					       ( void (*) () ) NULL,
					       KCMAP_ATT_END) ) == NULL )
	{
	    fprintf (stderr, "Error creating main colourmap\n");
	    a_prog_bug (function_name);
	}
	XtAddCallback (list, XtNcallback, cont_hg_cmap_cbk,
		       (XtPointer) cont_hg_cmap);
	setup_cont_hg_canvas (cont_hg_hist_pixcanvas, NULL, cont_hg_cmap);
	return (cont_hg_winpopup);
    }
    XtVaGetValues (cmap_cnv,
		   XkwNmonoPixCanvas, &cmap_pixcanvas,
		   XtNcolormap, &xcmap,
		   NULL);
    if ( ( dpy_handle = xc_get_dpy_handle (XtDisplay (cmap_cnv), xcmap) )
	== NULL )
    {
	fprintf (stderr, "Error getting display handle\n");
	a_prog_bug (function_name);
    }
    if ( ( cont_hg_cmap = kcmap_va_create (DEFAULT_COLOURMAP_NAME,
					   FULLCOLOUR_HOT_GAS_CMAP_SIZE, FALSE,
					   dpy_handle,
					   xc_alloc_colours, xc_free_colours,
					   xc_store_colours, xc_get_location2,
					   KCMAP_ATT_END) ) == NULL )
    {
	fprintf (stderr, "Error creating main colourmap\n");
	a_prog_bug (function_name);
    }
    XtAddCallback (list, XtNcallback, cont_hg_cmap_cbk,
		   (XtPointer) cont_hg_cmap);
    setup_cont_hg_canvas (cont_hg_hist_pixcanvas, cmap_pixcanvas,cont_hg_cmap);
    return (cont_hg_winpopup);
}   /*  End Function create_cont_hg_win  */


/*  Private functions follow  */

static void cont_hg_cmap_cbk (w, client_data, call_data)
/*  This is the continuous Hot Gas colourmap callback.
*/
Widget w;
XtPointer client_data;
XtPointer call_data;
{
    Kcolourmap cmap = (Kcolourmap) client_data;
    XawListReturnStruct *lr = (XawListReturnStruct *) call_data;
    extern KPixCanvas cont_hg_hist_pixcanvas;

    if ( !kcmap_change (cmap, lr->string, FULLCOLOUR_HOT_GAS_CMAP_SIZE,
			FALSE) )
    {
	fprintf (stderr, "Error changing colourmap\n");
    }
    compute_cont_hg_info ();
    kwin_resize (cont_hg_hist_pixcanvas, TRUE, 0, 0, 0, 0);
}   /*  End Function cont_hg_cmap_cbk   */

static void slider_cont_hg_cbk (w, client_data, call_data)
/*  This is the slider continuous Hot Gas callback.
*/
Widget w;
XtPointer client_data;
XtPointer call_data;
{
    extern KPixCanvas cont_hg_hist_pixcanvas;

    compute_cont_hg_info ();
    kwin_resize (cont_hg_hist_pixcanvas, TRUE, 0, 0, 0, 0);
}   /*  End Function slider_cont_hg_cbk   */

static void cont_hg_apply_cbk (w, client_data, call_data)
/*  This is the continuous Hot Gas apply callback.
*/
Widget w;
XtPointer client_data;
XtPointer call_data;
{
    extern KPixCanvas cont_hg_hist_pixcanvas;

    compute_cont_hg_info ();
    kwin_resize (cont_hg_hist_pixcanvas, TRUE, 0, 0, 0, 0);
    compute ();
}   /*  End Function cont_hg_apply_cbk   */
